@@ -25,7 +25,7 @@ for module in __allmodules__:
     __import__(module, globals(), locals(), [])
 
 import graph
-__allgraph__ = ["graphxy"]
+__allgraph__ = ["graphxy", "graphxyz"]
 for importfromgraph in __allgraph__:
     locals()[importfromgraph] = getattr(graph, importfromgraph)
 
