@@ -1,8 +1,7 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
-# Copyright (C) 2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2004-2006 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -18,13 +17,20 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
-__all__ = ["axis", "painter", "parter", "rater", "texter", "tick"]
-
-
-for module in __all__:
+__allmodules__ = ["painter", "parter", "rater", "texter", "tick"]
+for module in __allmodules__:
     __import__(module, globals(), locals(), [])
 
-from axis import linear, lin, logarithmic, log, linked, split, linkedsplit, bar, linkedbar, pathaxis
+import axis
+__allaxis__ = ["linear", "lin", "logarithmic", "log",
+               "bar", "nestedbar", "split",
+               "sizedlinear", "sizedlin", "autosizedlinear", "autosizedlin",
+               "anchoredaxis", "linkedaxis",
+               "anchoredpathaxis", "pathaxis"]
+for importfromaxis in __allaxis__:
+    locals()[importfromaxis] = getattr(axis, importfromaxis)
+
+__all__ = __allmodules__ + __allaxis__
