@@ -22,12 +22,21 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
+from __future__ import nested_scopes
+
 import math, warnings
-from pyx import attr, deco, style, color, unit, canvas, path
+from pyx import attr, deco, style, color, unit, canvas, path, mesh
 from pyx import text as textmodule
 
 builtinrange = range
 
+try:
+    sum([])
+except NameError:
+    # fallback implementation for Python 2.2 and below
+    def sum(list):
+        return reduce(lambda x, y: x+y, list, 0)
+
 try:
     enumerate([])
 except NameError:
@@ -121,7 +130,6 @@ _defaultprovider = {}
 
 def registerdefaultprovider(style, keys):
     """sets a style as a default creator for sharedata variables 'keys'"""
-    assert not len(style.needsdata), "currently we state, that a style should not depend on other sharedata variables"
     for key in keys:
         assert key in style.providesdata, "key not provided by style"
         # we might allow for overwriting the defaults, i.e. the following is not checked:
@@ -291,17 +299,19 @@ class range(_style):
                     if k != self.mask_value:
                         if k & (self.mask_dmin | self.mask_d):
                             mindata = []
-                            try:
-                                mindata.append(d[self.mask_value] - d[k])
-                            except:
-                                pass
+                            for value, delta in zip(d[self.mask_value], d[k]):
+                                try:
+                                    mindata.append(value-delta)
+                                except:
+                                    pass
                             graph.axes[a].adjustaxis(mindata)
                         if k & (self.mask_dmax | self.mask_d):
                             maxdata = []
-                            try:
-                                maxdata.append(d[self.mask_value] + d[k])
-                            except:
-                                pass
+                            for value, delta in zip(d[self.mask_value], d[k]):
+                                try:
+                                    maxdata.append(value+delta)
+                                except:
+                                    pass
                             graph.axes[a].adjustaxis(maxdata)
                         del d[k]
 
@@ -463,32 +473,17 @@ class symbol(_styleneedingpointpos):
             privatedata.symbol(graph, x_pt+0.5*width_pt, y_pt+0.5*height_pt, privatedata.size_pt, privatedata.symbolattrs)
 
 
-class line(_styleneedingpointpos):
+class _line(_styleneedingpointpos):
 
-    needsdata = ["vpos", "vposmissing", "vposavailable", "vposvalid"]
+    # this style is not a complete style, but it provides the basic functionality to
+    # create a line, which is cut at the graph boundaries (or at otherwise invalid points)
 
-    changelinestyle = attr.changelist([style.linestyle.solid,
-                                       style.linestyle.dashed,
-                                       style.linestyle.dotted,
-                                       style.linestyle.dashdotted])
-
-    defaultlineattrs = [changelinestyle]
-
-    def __init__(self, lineattrs=[]):
-        self.lineattrs = lineattrs
-
-    def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
-        if self.lineattrs is not None:
-            privatedata.lineattrs = attr.selectattrs(self.defaultlineattrs + self.lineattrs, selectindex, selecttotal)
-        else:
-            privatedata.lineattrs = None
-
-    def initdrawpoints(self, privatedata, sharedata, graph):
+    def initpointstopath(self, privatedata):
         privatedata.path = path.path()
         privatedata.linebasepoints = []
         privatedata.lastvpos = None
 
-    def addpointstopath(self, privatedata, sharedata):
+    def addpointstopath(self, privatedata):
         # add baselinepoints to privatedata.path
         if len(privatedata.linebasepoints) > 1:
             privatedata.path.append(path.moveto_pt(*privatedata.linebasepoints[0]))
@@ -498,17 +493,17 @@ class line(_styleneedingpointpos):
                 privatedata.path.append(path.lineto_pt(*privatedata.linebasepoints[1]))
         privatedata.linebasepoints = []
 
-    def drawpoint(self, privatedata, sharedata, graph, point):
+    def addpoint(self, privatedata, graphvpos_pt, vposavailable, vposvalid, vpos):
         # append linebasepoints
-        if sharedata.vposavailable:
+        if vposavailable:
             if len(privatedata.linebasepoints):
                 # the last point was inside the graph
-                if sharedata.vposvalid: # shortcut for the common case
-                    privatedata.linebasepoints.append(graph.vpos_pt(*sharedata.vpos))
+                if vposvalid: # shortcut for the common case
+                    privatedata.linebasepoints.append(graphvpos_pt(*vpos))
                 else:
                     # cut end
                     cut = 1
-                    for vstart, vend in zip(privatedata.lastvpos, sharedata.vpos):
+                    for vstart, vend in zip(privatedata.lastvpos, vpos):
                         newcut = None
                         if vend > 1:
                             # 1 = vstart + (vend - vstart) * cut
@@ -526,17 +521,17 @@ class line(_styleneedingpointpos):
                             cut = newcut
                     else:
                         cutvpos = []
-                        for vstart, vend in zip(privatedata.lastvpos, sharedata.vpos):
+                        for vstart, vend in zip(privatedata.lastvpos, vpos):
                             cutvpos.append(vstart + (vend - vstart) * cut)
-                        privatedata.linebasepoints.append(graph.vpos_pt(*cutvpos))
-                    self.addpointstopath(privatedata, sharedata)
+                        privatedata.linebasepoints.append(graphvpos_pt(*cutvpos))
+                    self.addpointstopath(privatedata)
             else:
                 # the last point was outside the graph
                 if privatedata.lastvpos is not None:
-                    if sharedata.vposvalid:
+                    if vposvalid:
                         # cut beginning
                         cut = 0
-                        for vstart, vend in zip(privatedata.lastvpos, sharedata.vpos):
+                        for vstart, vend in zip(privatedata.lastvpos, vpos):
                             newcut = None
                             if vstart > 1:
                                 # 1 = vstart + (vend - vstart) * cut
@@ -554,15 +549,15 @@ class line(_styleneedingpointpos):
                                 cut = newcut
                         else:
                             cutvpos = []
-                            for vstart, vend in zip(privatedata.lastvpos, sharedata.vpos):
+                            for vstart, vend in zip(privatedata.lastvpos, vpos):
                                 cutvpos.append(vstart + (vend - vstart) * cut)
-                            privatedata.linebasepoints.append(graph.vpos_pt(*cutvpos))
-                            privatedata.linebasepoints.append(graph.vpos_pt(*sharedata.vpos))
+                            privatedata.linebasepoints.append(graphvpos_pt(*cutvpos))
+                            privatedata.linebasepoints.append(graphvpos_pt(*vpos))
                     else:
                         # sometimes cut beginning and end
                         cutfrom = 0
                         cutto = 1
-                        for vstart, vend in zip(privatedata.lastvpos, sharedata.vpos):
+                        for vstart, vend in zip(privatedata.lastvpos, vpos):
                             newcutfrom = None
                             if vstart > 1:
                                 if vend > 1:
@@ -601,23 +596,119 @@ class line(_styleneedingpointpos):
                             if cutfrom < cutto:
                                 cutfromvpos = []
                                 cuttovpos = []
-                                for vstart, vend in zip(privatedata.lastvpos, sharedata.vpos):
+                                for vstart, vend in zip(privatedata.lastvpos, vpos):
                                     cutfromvpos.append(vstart + (vend - vstart) * cutfrom)
                                     cuttovpos.append(vstart + (vend - vstart) * cutto)
-                                privatedata.linebasepoints.append(graph.vpos_pt(*cutfromvpos))
-                                privatedata.linebasepoints.append(graph.vpos_pt(*cuttovpos))
-                                self.addpointstopath(privatedata, sharedata)
-            privatedata.lastvpos = sharedata.vpos[:]
+                                privatedata.linebasepoints.append(graphvpos_pt(*cutfromvpos))
+                                privatedata.linebasepoints.append(graphvpos_pt(*cuttovpos))
+                                self.addpointstopath(privatedata)
+            privatedata.lastvpos = vpos[:]
         else:
             if len(privatedata.linebasepoints) > 1:
-                self.addpointstopath(privatedata, sharedata)
+                self.addpointstopath(privatedata)
             privatedata.lastvpos = None
 
-    def donedrawpoints(self, privatedata, sharedata, graph):
+    def addinvalid(self, privatedata):
         if len(privatedata.linebasepoints) > 1:
-            self.addpointstopath(privatedata, sharedata)
-        if privatedata.lineattrs is not None and len(privatedata.path):
-            graph.stroke(privatedata.path, privatedata.lineattrs)
+            self.addpointstopath(privatedata)
+        privatedata.lastvpos = None
+
+    def donepointstopath(self, privatedata):
+        if len(privatedata.linebasepoints) > 1:
+            self.addpointstopath(privatedata)
+        return privatedata.path
+
+
+class line(_line):
+
+    needsdata = ["vpos", "vposmissing", "vposavailable", "vposvalid"]
+
+    changelinestyle = attr.changelist([style.linestyle.solid,
+                                       style.linestyle.dashed,
+                                       style.linestyle.dotted,
+                                       style.linestyle.dashdotted])
+
+    defaultlineattrs = [changelinestyle]
+
+    def __init__(self, lineattrs=[]):
+        self.lineattrs = lineattrs
+
+    def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
+        if self.lineattrs is not None:
+            privatedata.lineattrs = attr.selectattrs(self.defaultlineattrs + self.lineattrs, selectindex, selecttotal)
+        else:
+            privatedata.lineattrs = None
+
+    def initdrawpoints(self, privatedata, sharedata, graph):
+        self.initpointstopath(privatedata)
+
+    def drawpoint(self, privatedata, sharedata, graph, point):
+        self.addpoint(privatedata, graph.vpos_pt, sharedata.vposavailable, sharedata.vposvalid, sharedata.vpos)
+
+    def donedrawpoints(self, privatedata, sharedata, graph):
+        path = self.donepointstopath(privatedata)
+        if privatedata.lineattrs is not None and len(path):
+            graph.stroke(path, privatedata.lineattrs)
+
+    def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
+        if privatedata.lineattrs is not None:
+            graph.stroke(path.line_pt(x_pt, y_pt+0.5*height_pt, x_pt+width_pt, y_pt+0.5*height_pt), privatedata.lineattrs)
+
+
+class impulses(_styleneedingpointpos):
+
+    needsdata = ["vpos", "vposmissing", "vposavailable", "vposvalid", "poscolumnnames"]
+
+    defaultlineattrs = [line.changelinestyle]
+    defaultfrompathattrs = []
+
+    def __init__(self, lineattrs=[], fromvalue=0, frompathattrs=[], valueaxisindex=1):
+        self.lineattrs = lineattrs
+        self.fromvalue = fromvalue
+        self.frompathattrs = frompathattrs
+        self.valueaxisindex = valueaxisindex
+
+    def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
+        privatedata.insertfrompath = selectindex == 0
+        if self.lineattrs is not None:
+            privatedata.lineattrs = attr.selectattrs(self.defaultlineattrs + self.lineattrs, selectindex, selecttotal)
+        else:
+            privatedata.lineattrs = None
+
+    def adjustaxis(self, privatedata, sharedata, graph, columnname, data):
+        if self.fromvalue is not None:
+            try:
+                i = sharedata.poscolumnnames.index(columnname)
+            except ValueError:
+                pass
+            else:
+                if i == self.valueaxisindex:
+                    graph.axes[sharedata.poscolumnnames[i]].adjustaxis([self.fromvalue])
+
+    def initdrawpoints(self, privatedata, sharedata, graph):
+        privatedata.impulsescanvas = canvas.canvas()
+        if self.fromvalue is not None:
+            valueaxisname = sharedata.poscolumnnames[self.valueaxisindex]
+            privatedata.vfromvalue = graph.axes[valueaxisname].convert(self.fromvalue)
+            privatedata.vfromvaluecut = 0
+            if privatedata.vfromvalue < 0:
+                privatedata.vfromvalue = 0
+            if privatedata.vfromvalue > 1:
+                privatedata.vfromvalue = 1
+            if self.frompathattrs is not None and privatedata.insertfrompath:
+                graph.stroke(graph.axes[valueaxisname].vgridpath(privatedata.vfromvalue),
+                             self.defaultfrompathattrs + self.frompathattrs)
+        else:
+            privatedata.vfromvalue = 0
+
+    def drawpoint(self, privatedata, sharedata, graph, point):
+        if sharedata.vposvalid and privatedata.lineattrs is not None:
+            vpos = sharedata.vpos[:]
+            vpos[self.valueaxisindex] = privatedata.vfromvalue
+            privatedata.impulsescanvas.stroke(graph.vgeodesic(*(vpos + sharedata.vpos)), privatedata.lineattrs)
+
+    def donedrawpoints(self, privatedata, sharedata, graph):
+        graph.insert(privatedata.impulsescanvas)
 
     def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
         if privatedata.lineattrs is not None:
@@ -707,8 +798,14 @@ class text(_styleneedingpointpos):
 
     defaulttextattrs = [textmodule.halign.center, textmodule.vshift.mathaxis]
 
-    def __init__(self, textname="text", textdx=0*unit.v_cm, textdy=0.3*unit.v_cm, textattrs=[]):
+    def __init__(self, textname="text", dxname=None, dyname=None,
+                       dxunit=0.3*unit.v_cm, dyunit=0.3*unit.v_cm,
+                       textdx=0*unit.v_cm, textdy=0.3*unit.v_cm, textattrs=[]):
         self.textname = textname
+        self.dxname = dxname
+        self.dyname = dyname
+        self.dxunit = dxunit
+        self.dyunit = dyunit
         self.textdx = textdx
         self.textdy = textdy
         self.textattrs = textattrs
@@ -716,7 +813,16 @@ class text(_styleneedingpointpos):
     def columnnames(self, privatedata, sharedata, graph, columnnames):
         if self.textname not in columnnames:
             raise ValueError("column '%s' missing" % self.textname)
-        return [self.textname] + _styleneedingpointpos.columnnames(self, privatedata, sharedata, graph, columnnames)
+        names = [self.textname]
+        if self.dxname is not None:
+            if self.dxname not in columnnames:
+                raise ValueError("column '%s' missing" % self.dxname)
+            names.append(self.dxname)
+        if self.dyname is not None:
+            if self.dyname not in columnnames:
+                raise ValueError("column '%s' missing" % self.dyname)
+            names.append(self.dyname)
+        return names + _styleneedingpointpos.columnnames(self, privatedata, sharedata, graph, columnnames)
 
     def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
         if self.textattrs is not None:
@@ -725,8 +831,14 @@ class text(_styleneedingpointpos):
             privatedata.textattrs = None
 
     def initdrawpoints(self, privatedata, sharedata, grap):
-        privatedata.textdx_pt = unit.topt(self.textdx)
-        privatedata.textdy_pt = unit.topt(self.textdy)
+        if self.dxname is None:
+            privatedata.textdx_pt = unit.topt(self.textdx)
+        else:
+            privatedata.dxunit_pt = unit.topt(self.dxunit)
+        if self.dyname is None:
+            privatedata.textdy_pt = unit.topt(self.textdy)
+        else:
+            privatedata.dyunit_pt = unit.topt(self.dyunit)
 
     def drawpoint(self, privatedata, sharedata, graph, point):
         if privatedata.textattrs is not None and sharedata.vposvalid:
@@ -736,7 +848,15 @@ class text(_styleneedingpointpos):
             except:
                 pass
             else:
-                graph.text_pt(x_pt + privatedata.textdx_pt, y_pt + privatedata.textdy_pt, text, privatedata.textattrs)
+                if self.dxname is None:
+                    dx_pt = privatedata.textdx_pt
+                else:
+                    dx_pt = float(point[self.dxname]) * privatedata.dxunit_pt
+                if self.dyname is None:
+                    dy_pt = privatedata.textdy_pt
+                else:
+                    dy_pt = float(point[self.dyname]) * privatedata.dyunit_pt
+                graph.text_pt(x_pt + dx_pt, y_pt + dy_pt, text, privatedata.textattrs)
 
     def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
         raise RuntimeError("Style currently doesn't provide a graph key")
@@ -749,11 +869,12 @@ class arrow(_styleneedingpointpos):
     defaultlineattrs = []
     defaultarrowattrs = []
 
-    def __init__(self, linelength=0.25*unit.v_cm, arrowsize=0.15*unit.v_cm, lineattrs=[], arrowattrs=[], epsilon=1e-5):
+    def __init__(self, linelength=0.25*unit.v_cm, arrowsize=0.15*unit.v_cm, lineattrs=[], arrowattrs=[], arrowpos=0.5, epsilon=1e-5):
         self.linelength = linelength
         self.arrowsize = arrowsize
         self.lineattrs = lineattrs
         self.arrowattrs = arrowattrs
+        self.arrowpos = arrowpos
         self.epsilon = epsilon
 
     def columnnames(self, privatedata, sharedata, graph, columnnames):
@@ -791,10 +912,10 @@ class arrow(_styleneedingpointpos):
                 if point["size"] > self.epsilon:
                     dx = math.cos(angle*math.pi/180)
                     dy = math.sin(angle*math.pi/180)
-                    x1 = x_pt-0.5*dx*linelength_pt*size
-                    y1 = y_pt-0.5*dy*linelength_pt*size
-                    x2 = x_pt+0.5*dx*linelength_pt*size
-                    y2 = y_pt+0.5*dy*linelength_pt*size
+                    x1 = x_pt-self.arrowpos*dx*linelength_pt*size
+                    y1 = y_pt-self.arrowpos*dy*linelength_pt*size
+                    x2 = x_pt+(1-self.arrowpos)*dx*linelength_pt*size
+                    y2 = y_pt+(1-self.arrowpos)*dy*linelength_pt*size
                     privatedata.arrowcanvas.stroke(path.line_pt(x1, y1, x2, y2), privatedata.lineattrs +
                                                  [deco.earrow(privatedata.arrowattrs, size=self.arrowsize*size)])
 
@@ -809,8 +930,8 @@ class rect(_style):
 
     needsdata = ["vrange", "vrangeminmissing", "vrangemaxmissing"]
 
-    def __init__(self, palette=color.palette.Grey):
-        self.palette = palette
+    def __init__(self, gradient=color.gradient.Grey):
+        self.gradient = gradient
 
     def columnnames(self, privatedata, sharedata, graph, columnnames):
         if len(graph.axesnames) != 2:
@@ -846,7 +967,7 @@ class rect(_style):
             p.append(graph.vgeodesic_el(xvmax, yvmax, xvmin, yvmax))
             p.append(graph.vgeodesic_el(xvmin, yvmax, xvmin, yvmin))
             p.append(path.closepath())
-            privatedata.rectcanvas.fill(p, [self.palette.getcolor(point["color"])])
+            privatedata.rectcanvas.fill(p, [self.gradient.getcolor(point["color"])])
 
     def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
         raise RuntimeError("Style currently doesn't provide a graph key")
@@ -859,13 +980,14 @@ class histogram(_style):
     defaultlineattrs = [deco.stroked]
     defaultfrompathattrs = []
 
-    def __init__(self, lineattrs=[], steps=0, fromvalue=0, frompathattrs=[], fillable=0,
+    def __init__(self, lineattrs=[], steps=0, fromvalue=0, frompathattrs=[], fillable=0, rectkey=0,
                        autohistogramaxisindex=0, autohistogrampointpos=0.5, epsilon=1e-10):
         self.lineattrs = lineattrs
         self.steps = steps
         self.fromvalue = fromvalue
         self.frompathattrs = frompathattrs
         self.fillable = fillable # TODO: fillable paths might not properly be closed by straight lines on curved graph geometries
+        self.rectkey = rectkey
         self.autohistogramaxisindex = autohistogramaxisindex
         self.autohistogrampointpos = autohistogrampointpos
         self.epsilon = epsilon
@@ -898,6 +1020,8 @@ class histogram(_style):
                 min = data[0] - self.autohistogrampointpos * delta
                 max = data[-1] + (1-self.autohistogrampointpos) * delta
                 graph.axes[columnname].adjustaxis([min, max])
+        elif self.fromvalue is not None and columnname == sharedata.poscolumnnames[1-privatedata.rangeaxisindex]:
+            graph.axes[columnname].adjustaxis([self.fromvalue])
 
     def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
         privatedata.insertfrompath = selectindex == 0
@@ -1186,7 +1310,11 @@ class histogram(_style):
 
     def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
         if privatedata.lineattrs is not None:
-            graph.stroke(path.line_pt(x_pt, y_pt+0.5*height_pt, x_pt+width_pt, y_pt+0.5*height_pt), privatedata.lineattrs)
+            if self.rectkey:
+                p = path.rect_pt(x_pt, y_pt, width_pt, height_pt)
+            else:
+                p = path.line_pt(x_pt, y_pt+0.5*height_pt, x_pt+width_pt, y_pt+0.5*height_pt)
+            graph.draw(p, privatedata.lineattrs)
 
 
 class barpos(_style):
@@ -1349,7 +1477,7 @@ class bar(_style):
 
     needsdata = ["vbarrange"]
 
-    defaultbarattrs = [color.palette.Rainbow, deco.stroked([color.grey.black])]
+    defaultbarattrs = [color.gradient.Rainbow, deco.stroked([color.grey.black])]
 
     def __init__(self, barattrs=[]):
         self.barattrs = barattrs
@@ -1435,3 +1563,313 @@ class changebar(bar):
 
     def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
         raise RuntimeError("Style currently doesn't provide a graph key")
+
+
+class gridpos(_style):
+
+    needsdata = ["vpos", "vposmissing", "vposavailable", "vposvalid"]
+    providesdata = ["values1", "values2", "data12", "data21", "index1", "index2"]
+
+    def __init__(self, index1=0, index2=1, epsilon=1e-10):
+        self.index1 = index1
+        self.index2 = index2
+        self.epsilon = epsilon
+
+    def initdrawpoints(self, privatedata, sharedata, graph):
+        sharedata.index1 = self.index1
+        sharedata.index2 = self.index2
+        sharedata.values1 = {}
+        sharedata.values2 = {}
+        sharedata.data12 = {}
+        sharedata.data21 = {}
+
+    def drawpoint(self, privatedata, sharedata, graph, point):
+        if sharedata.vposavailable:
+            sharedata.value1 = sharedata.vpos[self.index1]
+            sharedata.value2 = sharedata.vpos[self.index2]
+            if not sharedata.values1.has_key(sharedata.value1):
+                for hasvalue in sharedata.values1.keys():
+                    if hasvalue - self.epsilon <= sharedata.value1 <= hasvalue + self.epsilon:
+                        sharedata.value1 = hasvalue
+                        break
+                else:
+                    sharedata.values1[sharedata.value1] = 1
+            if not sharedata.values2.has_key(sharedata.value2):
+                for hasvalue in sharedata.values2.keys():
+                    if hasvalue - self.epsilon <= sharedata.value2 <= hasvalue + self.epsilon:
+                        sharedata.value2 = hasvalue
+                        break
+                else:
+                    sharedata.values2[sharedata.value2] = 1
+            data = sharedata.vposavailable, sharedata.vposvalid, sharedata.vpos[:]
+            sharedata.data12.setdefault(sharedata.value1, {})[sharedata.value2] = data
+            sharedata.data21.setdefault(sharedata.value2, {})[sharedata.value1] = data
+
+registerdefaultprovider(gridpos(), gridpos.providesdata)
+
+
+class grid(_line, _style):
+
+    needsdata = ["values1", "values2", "data12", "data21"]
+
+    defaultgridattrs = [line.changelinestyle]
+
+    def __init__(self, gridlines1=1, gridlines2=1, gridattrs=[]):
+        self.gridlines1 = gridlines1
+        self.gridlines2 = gridlines2
+        self.gridattrs = gridattrs
+
+    def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
+        if self.gridattrs is not None:
+            privatedata.gridattrs = attr.selectattrs(self.defaultgridattrs + self.gridattrs, selectindex, selecttotal)
+        else:
+            privatedata.gridattrs = None
+
+    def donedrawpoints(self, privatedata, sharedata, graph):
+        values1 = sharedata.values1.keys()
+        values1.sort()
+        values2 = sharedata.values2.keys()
+        values2.sort()
+        if self.gridlines1:
+            for value2 in values2:
+                data1 = sharedata.data21[value2]
+                self.initpointstopath(privatedata)
+                for value1 in values1:
+                    try:
+                        data = data1[value1]
+                    except KeyError:
+                        self.addinvalid(privatedata)
+                    else:
+                        self.addpoint(privatedata, graph.vpos_pt, *data)
+                p = self.donepointstopath(privatedata)
+                if len(p):
+                    graph.stroke(p, privatedata.gridattrs)
+        if self.gridlines2:
+            for value1 in values1:
+                data2 = sharedata.data12[value1]
+                self.initpointstopath(privatedata)
+                for value2 in values2:
+                    try:
+                        data = data2[value2]
+                    except KeyError:
+                        self.addinvalid(privatedata)
+                    else:
+                        self.addpoint(privatedata, graph.vpos_pt, *data)
+                p = self.donepointstopath(privatedata)
+                if len(p):
+                    graph.stroke(p, privatedata.gridattrs)
+
+
+class surface(_style):
+
+    needsdata = ["values1", "values2", "data12", "data21"]
+
+    def __init__(self, colorname="color", gradient=color.gradient.Grey, mincolor=None, maxcolor=None,
+                       gridlines1=0.05, gridlines2=0.05, gridcolor=None,
+                       backcolor=color.gray.black):
+        self.colorname = colorname
+        self.gradient = gradient
+        self.mincolor = mincolor
+        self.maxcolor = maxcolor
+        self.gridlines1 = gridlines1
+        self.gridlines2 = gridlines2
+        self.gridcolor = gridcolor
+        self.backcolor = backcolor
+
+        colorspacestring = gradient.getcolor(0).colorspacestring()
+        if self.gridcolor is not None and self.gridcolor.colorspacestring() != colorspacestring:
+            raise RuntimeError("colorspace mismatch (gradient/grid)")
+        if self.backcolor is not None and self.backcolor.colorspacestring() != colorspacestring:
+            raise RuntimeError("colorspace mismatch (gradient/back)")
+
+    def midvalue(self, v1, v2, v3, v4):
+        return [0.25*sum(values) for values in zip(v1, v2, v3, v4)]
+
+    def midcolor(self, c1, c2, c3, c4):
+        return 0.25*(c1+c2+c3+c4)
+
+    def lightning(self, angle, zindex):
+        if angle < 0 and self.backcolor is not None:
+            return self.backcolor
+        return self.gradient.getcolor(0.7-0.4*abs(angle)+0.1*zindex)
+
+    def columnnames(self, privatedata, sharedata, graph, columnnames):
+        privatedata.colorize = self.colorname in columnnames
+        if privatedata.colorize:
+            return [self.colorname]
+        return []
+
+    def initdrawpoints(self, privatedata, sharedata, graph):
+        privatedata.colors = {}
+        privatedata.mincolor = privatedata.maxcolor = None
+
+    def drawpoint(self, privatedata, sharedata, graph, point):
+        if privatedata.colorize:
+            try:
+                color = point[self.colorname] + 0
+            except:
+                pass
+            else:
+                privatedata.colors.setdefault(sharedata.value1, {})[sharedata.value2] = color
+                if privatedata.mincolor is None or color < privatedata.mincolor:
+                    privatedata.mincolor = color
+                if privatedata.mincolor is None or privatedata.maxcolor < color:
+                    privatedata.maxcolor = color
+
+    def donedrawpoints(self, privatedata, sharedata, graph):
+        v1 = [0]*len(graph.axesnames)
+        v2 = [0]*len(graph.axesnames)
+        v3 = [0]*len(graph.axesnames)
+        v4 = [0]*len(graph.axesnames)
+        v1[sharedata.index2] = 0.5
+        v2[sharedata.index1] = 0.5
+        v3[sharedata.index1] = 0.5
+        v3[sharedata.index2] = 1
+        v4[sharedata.index1] = 1
+        v4[sharedata.index2] = 0.5
+        sortElements = [-graph.vzindex(*v1),
+                        -graph.vzindex(*v2),
+                        -graph.vzindex(*v3),
+                        -graph.vzindex(*v4)]
+
+        values1 = sharedata.values1.keys()
+        values1.sort()
+        v1 = [0]*len(graph.axesnames)
+        v2 = [0]*len(graph.axesnames)
+        v1[sharedata.index1] = -1
+        v2[sharedata.index1] = 1
+        sign = 1
+        if graph.vzindex(*v1) < graph.vzindex(*v2):
+            values1.reverse()
+            sign *= -1
+            sortElements = [sortElements[3], sortElements[1], sortElements[2], sortElements[0]]
+
+        values2 = sharedata.values2.keys()
+        values2.sort()
+        v1 = [0]*len(graph.axesnames)
+        v2 = [0]*len(graph.axesnames)
+        v1[sharedata.index2] = -1
+        v2[sharedata.index2] = 1
+        if graph.vzindex(*v1) < graph.vzindex(*v2):
+            values2.reverse()
+            sign *= -1
+            sortElements = [sortElements[0], sortElements[2], sortElements[1], sortElements[3]]
+
+        sortElements = [(zindex, i) for i, zindex in enumerate(sortElements)]
+        sortElements.sort()
+
+        if self.mincolor is not None:
+            mincolor = self.mincolor
+        if self.maxcolor is not None:
+            maxcolor = self.maxcolor
+        nodes = []
+        elements = []
+        for value1a, value1b in zip(values1[:-1], values1[1:]):
+            for value2a, value2b in zip(values2[:-1], values2[1:]):
+                try:
+                    available1, valid1, v1 = sharedata.data12[value1a][value2a]
+                    available2, valid2, v2 = sharedata.data12[value1a][value2b]
+                    available3, valid3, v3 = sharedata.data12[value1b][value2a]
+                    available4, valid4, v4 = sharedata.data12[value1b][value2b]
+                except KeyError:
+                    continue
+                if not available1 or not available2 or not available3 or not available4:
+                    continue
+                if not valid1 or not valid2 or not valid3 or not valid4:
+                    warnings.warn("surface elements partially outside of the graph are (currently) skipped completely")
+                    continue
+                def shrink(index, v1, v2, by):
+                    v1 = v1[:]
+                    v2 = v2[:]
+                    for i in builtinrange(3):
+                        if i != index:
+                            v1[i], v2[i] = v1[i] + by*(v2[i]-v1[i]), v2[i] + by*(v1[i]-v2[i])
+                    return v1, v2
+                v1f, v2f, v3f, v4f = v1, v2, v3, v4
+                if self.gridcolor is not None and self.gridlines1:
+                    v1, v2 = shrink(sharedata.index1, v1, v2, self.gridlines1)
+                    v3, v4 = shrink(sharedata.index1, v3, v4, self.gridlines1)
+                if self.gridcolor is not None and self.gridlines2:
+                    v1, v3 = shrink(sharedata.index2, v1, v3, self.gridlines2)
+                    v2, v4 = shrink(sharedata.index2, v2, v4, self.gridlines2)
+                v5 = self.midvalue(v1, v2, v3, v4)
+                x1_pt, y1_pt = graph.vpos_pt(*v1)
+                x2_pt, y2_pt = graph.vpos_pt(*v2)
+                x3_pt, y3_pt = graph.vpos_pt(*v3)
+                x4_pt, y4_pt = graph.vpos_pt(*v4)
+                x5_pt, y5_pt = graph.vpos_pt(*v5)
+                if privatedata.colorize:
+                    def colorfromgradient(c):
+                        return self.gradient.getcolor((c - privatedata.mincolor) /
+                                                      float(privatedata.maxcolor - privatedata.mincolor))
+                    c1 = privatedata.colors[value1a][value2a]
+                    c2 = privatedata.colors[value1a][value2b]
+                    c3 = privatedata.colors[value1b][value2a]
+                    c4 = privatedata.colors[value1b][value2b]
+                    c5 = self.midcolor(c1, c2, c3, c4)
+                    c1a = c1b = colorfromgradient(c1)
+                    c2a = c2c = colorfromgradient(c2)
+                    c3b = c3d = colorfromgradient(c3)
+                    c4c = c4d = colorfromgradient(c4)
+                    c5a = c5b = c5c = c5d = colorfromgradient(c5)
+                    if self.backcolor is not None and sign*graph.vangle(*(v1+v2+v5)) < 0:
+                        c1a = c2a = c5a = self.backcolor
+                    if self.backcolor is not None and sign*graph.vangle(*(v3+v1+v5)) < 0:
+                        c3b = c1b = c5b = self.backcolor
+                    if self.backcolor is not None and sign*graph.vangle(*(v2+v4+v5)) < 0:
+                        c2c = c4c = c5c = self.backcolor
+                    if self.backcolor is not None and sign*graph.vangle(*(v4+v3+v5)) < 0:
+                        c4d = c3d = c5d = self.backcolor
+                else:
+                    zindex = graph.vzindex(*v5)
+                    c1a = c2a = c5a = self.lightning(sign*graph.vangle(*(v1+v2+v5)), zindex)
+                    c3b = c1b = c5b = self.lightning(sign*graph.vangle(*(v3+v1+v5)), zindex)
+                    c2c = c4c = c5c = self.lightning(sign*graph.vangle(*(v2+v4+v5)), zindex)
+                    c4d = c3d = c5d = self.lightning(sign*graph.vangle(*(v4+v3+v5)), zindex)
+                for zindex, i in sortElements:
+                    if i == 0:
+                        elements.append(mesh.element((mesh.node_pt((x1_pt, y1_pt), c1a),
+                                                      mesh.node_pt((x2_pt, y2_pt), c2a),
+                                                      mesh.node_pt((x5_pt, y5_pt), c5a))))
+                        if self.gridcolor is not None and self.gridlines2:
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v1f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v2), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v1), self.gridcolor))))
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v1f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v2), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v2f), self.gridcolor))))
+                    elif i == 1:
+                        elements.append(mesh.element((mesh.node_pt((x3_pt, y3_pt), c3b),
+                                                      mesh.node_pt((x1_pt, y1_pt), c1b),
+                                                      mesh.node_pt((x5_pt, y5_pt), c5b))))
+                        if self.gridcolor is not None and self.gridlines1:
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v1f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v3), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v1), self.gridcolor))))
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v1f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v3), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v3f), self.gridcolor))))
+                    elif i == 2:
+                        elements.append(mesh.element((mesh.node_pt((x2_pt, y2_pt), c2c),
+                                                      mesh.node_pt((x4_pt, y4_pt), c4c),
+                                                      mesh.node_pt((x5_pt, y5_pt), c5c))))
+                        if self.gridcolor is not None and self.gridlines1:
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v2f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v4), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v2), self.gridcolor))))
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v2f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v4), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v4f), self.gridcolor))))
+                    elif i == 3:
+                        elements.append(mesh.element((mesh.node_pt((x4_pt, y4_pt), c4d),
+                                                      mesh.node_pt((x3_pt, y3_pt), c3d),
+                                                      mesh.node_pt((x5_pt, y5_pt), c5d))))
+                        if self.gridcolor is not None and self.gridlines2:
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v3f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v4), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v3), self.gridcolor))))
+                            elements.append(mesh.element((mesh.node_pt(graph.vpos_pt(*v3f), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v4), self.gridcolor),
+                                                          mesh.node_pt(graph.vpos_pt(*v4f), self.gridcolor))))
+        m = mesh.mesh(elements, check=0)
+        graph.insert(m)
