@@ -82,7 +82,7 @@ class _invalid:
 
     def invalid2(self, other):
         self.invalid1()
-    __cmp__ = __add__ = __iadd__ = __sub__ = __isub__ = __mul__ = __imul__ = __div__ = __idiv__ = invalid2
+    __cmp__ = __add__ = __iadd__ = __sub__ = __isub__ = __mul__ = __imul__ = __div__ = __truediv__ = __idiv__ = invalid2
 
 invalid = _invalid()
 
@@ -913,6 +913,8 @@ class normsubpath:
 
     def at_pt(self, params):
         """return coordinates at params in pts"""
+        if not self.normsubpathitems and self.skippedline:
+            return [self.skippedline.atbegin_pt()]*len(params)
         result = [None] * len(params)
         for normsubpathitemindex, (indices, params) in self._distributeparams(params).items():
             for index, point_pt in zip(indices, self.normsubpathitems[normsubpathitemindex].at_pt(params)):
@@ -1162,6 +1164,8 @@ class normsubpath:
 
     def _paramtoarclen_pt(self, params):
         """return a tuple of arc lengths and the total arc length in pts"""
+        if not self.normsubpathitems:
+            return [0] * len(params), 0
         result = [None] * len(params)
         totalarclen_pt = 0
         distributeparams = self._distributeparams(params)
