@@ -2,7 +2,7 @@
 #
 #
 # Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
-# Copyright (C) 2003-2004,2006 Michael Schindler <m-schindler@users.sourceforge.net>
+# Copyright (C) 2003-2007 Michael Schindler <m-schindler@users.sourceforge.net>
 # Copyright (C) 2002-2006 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
@@ -243,13 +243,14 @@ class _texmessageemptylines(texmessage):
 class _texmessageload(texmessage):
     """validates inclusion of arbitrary files
     - the matched pattern is "(<filename> <arbitrary other stuff>)", where
-      <fielname> is a readable file and other stuff can be anything
+      <filename> is a readable file and other stuff can be anything
+    - If the filename is enclosed in double quotes, it may contain blank space.
     - "(" and ")" must be used consistent (otherwise this validator just does nothing)
     - this is not always wanted, but we just assume that file inclusion is fine"""
 
     __implements__ = _Itexmessage
 
-    pattern = re.compile(r"\((?P<filename>[^()\s\n]+)(?P<additional>[^()]*)\)")
+    pattern = re.compile(r"\([\"]?(?P<filename>(?:(?<!\")[^()\s\n]+(?!\"))|[^()\"\n]+)[\"]?(?P<additional>[^()]*)\)")
 
     def baselevels(self, s, maxlevel=1, brackets="()"):
         """strip parts of a string above a given bracket level
@@ -373,6 +374,7 @@ class texmessagepattern(texmessage):
 
 texmessage.fontwarning = texmessagepattern(re.compile(r"^LaTeX Font Warning: .*$(\n^\(Font\).*$)*", re.MULTILINE), "ignoring font warning")
 texmessage.boxwarning = texmessagepattern(re.compile(r"^(Overfull|Underfull) \\[hv]box.*$(\n^..*$)*\n^$\n", re.MULTILINE), "ignoring overfull/underfull box warning")
+texmessage.rerunwarning = texmessagepattern(re.compile(r"^(LaTeX Warning: Label\(s\) may have changed\. Rerun to get cross-references right\s*\.)$", re.MULTILINE), "ignoring rerun warning")
 
 
 
@@ -760,7 +762,7 @@ class texrunner:
     defaulttexmessagesstart = [texmessage.start]
     defaulttexmessagesdocclass = [texmessage.load]
     defaulttexmessagesbegindoc = [texmessage.load, texmessage.noaux]
-    defaulttexmessagesend = [texmessage.end, texmessage.fontwarning]
+    defaulttexmessagesend = [texmessage.end, texmessage.fontwarning, texmessage.rerunwarning]
     defaulttexmessagesdefaultpreamble = [texmessage.load]
     defaulttexmessagesdefaultrun = [texmessage.loaddef, texmessage.graphicsload,
                                     texmessage.fontwarning, texmessage.boxwarning]
@@ -791,7 +793,7 @@ class texrunner:
         self.lfs = lfs
         self.docclass = docclass
         self.docopt = docopt
-        self.usefiles = usefiles
+        self.usefiles = usefiles[:]
         self.fontmaps = fontmaps
         self.waitfortex = waitfortex
         self.showwaitfortex = showwaitfortex
@@ -806,12 +808,12 @@ class texrunner:
         self.dvidebug = dvidebug
         self.errordebug = errordebug
         self.pyxgraphics = pyxgraphics
-        self.texmessagesstart = texmessagesstart
-        self.texmessagesdocclass = texmessagesdocclass
-        self.texmessagesbegindoc = texmessagesbegindoc
-        self.texmessagesend = texmessagesend
-        self.texmessagesdefaultpreamble = texmessagesdefaultpreamble
-        self.texmessagesdefaultrun = texmessagesdefaultrun
+        self.texmessagesstart = texmessagesstart[:]
+        self.texmessagesdocclass = texmessagesdocclass[:]
+        self.texmessagesbegindoc = texmessagesbegindoc[:]
+        self.texmessagesend = texmessagesend[:]
+        self.texmessagesdefaultpreamble = texmessagesdefaultpreamble[:]
+        self.texmessagesdefaultrun = texmessagesdefaultrun[:]
 
         self.texruns = 0
         self.texdone = 0
